/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define WIDTH 800
#define HEIGHT 600
#define FPS (200)
#define FRAME_INTERVAL (1000/FPS)

#include <glib.h>

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>

#define SWAP(x, y) do { \
	x ^= y; y ^= x; x ^= y; \
	} while (0)

#define IS_CENTER(cx, cy, x, y) \
	((x + WIDTH/2 == cx) && (y + HEIGHT/2 == cy))

GArray *points;
GPtrArray *names;

void
InsertLine (GArray *points, SDL_Rect *src, SDL_Rect *dst)
{
  SDL_Rect rect;
  int inc, err, thre, swap;
  int x1, y1, x2, y2;
  int x, y;
  rect.w = WIDTH;
  rect.h = HEIGHT;
  err = 0;
  swap = 0;
  x1 = src->x;
  y1 = src->y;
  x2 = dst->x;
  y2 = dst->y;
  inc = y2 - y1;
  thre = x2 - x1;
  if (ABS (inc) > ABS (thre))
  {
    SWAP (inc, thre);
    SWAP (x1, y1);
    SWAP (x2, y2);
    swap = 1;
  }
  for (y = y1, x = x1; (x2 < x1) ? (x >= x2) : (x <= x2); (x2 < x1) ? x-- : x++)
  {
    rect.x = (swap ? y : x);
    rect.y = (swap ? x : y);
    g_array_append_val (points, rect);
    err += ABS (inc);
    if (err >= ABS (thre))
    {
      err -= ABS (thre);
      y += (inc < 0) ? -1 : 1;
    }
  }
}

void
ReadPoints (char *filename)
{
  FILE *file;
  char *buffer;
  char *next;
  size_t len;
  ssize_t r;
  int i = 0;
  SDL_Rect last;
  SDL_Rect rect;
  file = fopen (filename, "r");
  points = g_array_new (FALSE, TRUE, sizeof (SDL_Rect));
  names = g_ptr_array_new ();
  buffer = NULL;
  len = 0;
  rect.x = rect.y = rect.w = rect.h = 0;
  last = rect;
  while (!feof (file))
  {
    r = getline (&buffer, &len, file);
    buffer[r - 1] = '\0';
    rect.x = strtol (buffer, &next, 0);
    rect.y = strtol (next+1, &next, 0);
    strtol (next, &next, 0);
    if (i > 0)
      InsertLine (points, &last, &rect);
    while (isspace (*next)) next++;
    g_ptr_array_add (names, strdup (next));
    last = rect;
    i++;
  }
  fclose (file);
}

void
ShowPoint (SDL_Surface *screen, SDL_Surface *image, SDL_Rect center, double scale)
{
  center.x = (center.x - WIDTH/2) * scale;
  center.y = (center.y - HEIGHT/2) * scale;
  center.w = WIDTH;
  center.h = HEIGHT;
  SDL_BlitSurface (image, &center, screen, NULL);
  SDL_UpdateRect (screen, 0, 0, 0, 0);
}

int
main (int argc, char **argv)
{
  SDL_Surface *screen;
  SDL_Surface *image;
  SDL_Rect rect;
  SDL_Event event;
  Uint32 last, now, deslast, start;
  int i, des, desl;
  ReadPoints ("pro-gnu");
  SDL_Init (SDL_INIT_VIDEO | SDL_INIT_TIMER);
  screen = SDL_SetVideoMode (800, 600, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
  image = IMG_Load ("/home/cascardo/fotos/debconf.jpg");
  start = deslast = last = SDL_GetTicks ();
  desl = des = i = 0;
  while (1)
  {
    if (SDL_PollEvent (&event))
    {
      if (event.type == SDL_KEYDOWN)
        break;
    }
    now = SDL_GetTicks ();
    /* skip */
    if (now > deslast + 1000)
    {
      printf ("%f %f\n", (double) (now - start) / (double) des, (double) (now - deslast) / (double) (des - desl));
      desl = des;
      deslast = now;
    }
    while (now > last + FRAME_INTERVAL)
    {
      last += FRAME_INTERVAL;
      i++;
    }
    if (i > points->len)
      i = 0;
    rect = g_array_index (points, SDL_Rect, i);
    ShowPoint (screen, image, rect, 1.0);
    SDL_Delay (FRAME_INTERVAL - (now - last));
    i++;
    des++;
  }
  SDL_FreeSurface (image);
  SDL_Quit ();
  g_array_free (points, TRUE);
  for (i = 0; i < names->len; i++)
    free (g_ptr_array_index (names, i));
  g_ptr_array_free (names, TRUE);
  return 0;
}
